/*=========================================================================
  Program:   DTR
  Module:    vtkContourTopogramProjectionFilter.h
  Description:
    This filter projects polydata points using a topogram transform.
    Projection geometry is defined by the input transform matrix.
    Reference transform is used to trasnform points back into LPS and
    is expect to be the matching projection trasform without correction
    offsets and rotations.

  gfattori Feb 2022
=========================================================================*/

#ifndef VTKCONTOURTOPOGRAMPROJECTIONFILTER_H
#define VTKCONTOURTOPOGRAMPROJECTIONFILTER_H

#include "vtkFiltersCoreModule.h" // For export macro
#include "vtkPolyDataAlgorithm.h"
#include "vtkMatrix4x4.h"
#include "vtkTransform.h"

//#include "itkMacro.h"
#include "DRTMetaInformation.h"

class VTKFILTERSCORE_EXPORT vtkContourTopogramProjectionFilter :
        public vtkPolyDataAlgorithm
{

public:
    vtkTypeMacro(vtkContourTopogramProjectionFilter, vtkPolyDataAlgorithm);
    void PrintSelf(ostream& os, vtkIndent indent) override;


    static vtkContourTopogramProjectionFilter * New();

    void SetReferenceTransformMatrix( vtkMatrix4x4 *);
    void SetTransformMatrix( vtkMatrix4x4 *);

    vtkGetObjectMacro(m_Transform, vtkTransform);
    vtkGetObjectMacro(m_RefTransform, vtkTransform);

    itkSetEnumMacro(ProjectionOrientation, itk::tProjOrientationType);
    itkGetEnumMacro(ProjectionOrientation, itk::tProjOrientationType);

    itkSetEnumMacro(PatientOrientation, itk::tPatOrientation);
    itkGetEnumMacro(PatientOrientation, itk::tPatOrientation);

    void SetLPStoProjectionLPSGeometryOffset (const double * );
    void SetSCD(const double dVal);
    void SetImportOffsetLPS(const double *);
    void SetPanelOffset(const double dVal);

    void cleanUpFilter();

protected:
     vtkContourTopogramProjectionFilter();
     ~vtkContourTopogramProjectionFilter() override;

     int RequestData(vtkInformation* request, vtkInformationVector** inputVector,
            vtkInformationVector* outputVector) override;
     int FillInputPortInformation(int port, vtkInformation* info) override;

     itk::tProjOrientationType
     m_ProjectionOrientation;

     itk::tPatOrientation
     m_PatientOrientation;

private:
     vtkContourTopogramProjectionFilter (const vtkContourTopogramProjectionFilter &) = delete;
     void operator=(const vtkContourTopogramProjectionFilter&) = delete;


     double
        dProjectionLPSOffset[3],
        dImportOffsetLPS[3],
        dSCD,
        dPOffset;

     vtkTransform *m_RefTransform;
     vtkTransform *m_Transform;

};


#endif
